%***************************
% Written by Farinaz Edalat
% May 9, 2005
%***************************

%******************************************************
% Description: Power-Spectral-Density (PSD)
% (1) Find PSD of input and output of channel
% (2) Find frequency response of simulated channel
% (3) Compare channel PSD obtained via (2) and (1) <-- they must lie over
% each other!
%******************************************************

% (1) Load in the parameters and calculate PSD of input/output
x = loadsig_cppsim('test.tr0'); % load file from C++ sim
t = evalsig(x,'TIME'); % simulation time samples
Ts = t(200)-t(199); % simulation sample period

% Channel input signal
x_i = evalsig(x,'x_i');  
x_q = evalsig(x,'x_q');
signal_x = x_i + j*x_q;
signal_x = signal_x - mean(signal_x);

% Channel output signal
y_i = evalsig(x,'y_i');
y_q = evalsig(x,'y_q');
signal_y = y_i + j*y_q;
signal_y = signal_y - mean(signal_y);

% PSD of signal_x
[Pxx,f] = psd(sqrt(Ts)*signal_x,2^16,1/Ts,2^16);
Pxx_cen = fftshift(Pxx);
fx = f - 1.0/(2.0*Ts);
%f = f - f(length(f)/2+1); %gives same answer as above line

% PSD of signal_y
[Pyy,f] = psd(sqrt(Ts)*signal_y,2^16,1/Ts,2^16);
Pyy_cen = fftshift(Pyy);
fy = f - 1.0/(2.0*Ts);

Pxx_db = 10*log10(Pxx_cen);
Pyy_db = 10*log10(Pyy_cen);

% (2) Find channel frequency response 
load -ascii a_i.dat; % these were saved in nannel_tdl.m
load -ascii a_q.dat;
h = a_i + j*a_q;

N = 2^15;
n = [1:N];
k = [0:N-1];
omega_dt = 2*pi*k/N;
omega_ct = omega_dt ./ Ts;
freq = omega_ct ./ (2*pi);
freq_cen = freq - 1/(2*Ts);
H = fftshift(fft(h,N));

% (3) Comparison of channel PSD by 2 methods
P_diff = Pyy_db - Pxx_db; %Pyy = Pxx + Pchannel (P_diff = Pchannel)

figure(1)
plot(fy, P_diff,'r',freq_cen,20*log10(abs(H)),'b');
xlabel('Frequency Offset from Carrier (Hz)');
ylabel('Channel PSD (dBc/Hz)');
grid on;

